#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;
using LibSVM;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [Item("SupportVectorParameterTuningEvaluator", "")]
  [StorableClass]
  public class SupportVectorParameterTuningEvaluator : InstrumentedOperator, ISingleObjectiveEvaluator {
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }

    [StorableConstructor]
    protected SupportVectorParameterTuningEvaluator(bool deserializing) : base(deserializing) { }
    protected SupportVectorParameterTuningEvaluator(SupportVectorParameterTuningEvaluator original, Cloner cloner) : base(original, cloner) { }

    public SupportVectorParameterTuningEvaluator()
      : base() {
      Parameters.Add(new LookupParameter<RealVector>("Point"));
      Parameters.Add(new LookupParameter<IRegressionProblemData>("ProblemData"));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SupportVectorParameterTuningEvaluator(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var problemData = (IRegressionProblemData)Parameters["ProblemData"].ActualValue;
      var p = (RealVector)Parameters["Point"].ActualValue;
      var parameters = new svm_parameter {
        C = Math.Exp(p[0]),
        gamma = Math.Exp(p[1]),
        p = Math.Exp(p[2]),
        eps = 0.001,
        svm_type = svm_parameter.EPSILON_SVR,
        kernel_type = svm_parameter.RBF
      };

      var cvLoss = SupportVectorMachineUtil.CrossValidate(problemData, parameters, 5, false);
      QualityParameter.ActualValue = new DoubleValue(cvLoss);
      return base.InstrumentedApply();
    }

  }
}
